---
# generated by https://github.com/hashicorp/terraform-plugin-docs
page_title: "harvester_virtualmachine Resource - terraform-provider-harvester"
subcategory: ""
description: |-
  
---

# harvester_virtualmachine (Resource)



## Example Usage

```terraform
resource "harvester_virtualmachine" "k3os" {
  count     = 3
  name      = "k3os-${count.index}"
  namespace = "default"

  description = "test k3os iso image"
  tags = {
    ssh-user = "rancher"
  }

  cpu    = 4
  memory = "4Gi"

  efi         = true
  secure_boot = false

  network_interface {
    name         = "nic-1"
    network_name = harvester_network.mgmt-vlan1.id
  }

  disk {
    name       = "cdrom-disk"
    type       = "cd-rom"
    size       = "10Gi"
    bus        = "sata"
    boot_order = 2

    image       = harvester_image.k3os.id
    auto_delete = true
  }

  disk {
    name       = "rootdisk"
    type       = "disk"
    size       = "10Gi"
    bus        = "virtio"
    boot_order = 1
  }

  input {
    name = "tablet"
    type = "tablet"
    bus  = "usb"
  }
}


resource "harvester_virtualmachine" "ubuntu20" {
  name                 = "ubuntu20"
  namespace            = "default"
  restart_after_update = true

  description = "test ubuntu20 raw image"
  tags = {
    ssh-user = "ubuntu"
  }

  cpu    = 2
  memory = "2Gi"

  efi         = true
  secure_boot = true

  run_strategy    = "RerunOnFailure"
  hostname        = "ubuntu20"
  reserved_memory = "100Mi"
  machine_type    = "q35"

  network_interface {
    name           = "nic-1"
    wait_for_lease = true
  }

  disk {
    name       = "rootdisk"
    type       = "disk"
    size       = "10Gi"
    bus        = "virtio"
    boot_order = 1

    image       = harvester_image.ubuntu20.id
    auto_delete = true
  }

  disk {
    name        = "emptydisk"
    type        = "disk"
    size        = "20Gi"
    bus         = "virtio"
    auto_delete = true
  }

  cloudinit {
    user_data_secret_name    = harvester_cloudinit_secret.cloud-config-ubuntu20.name
    network_data_secret_name = harvester_cloudinit_secret.cloud-config-ubuntu20.name
  }
}

resource "harvester_virtualmachine" "opensuse154" {
  name                 = "opensuse154"
  namespace            = "default"
  restart_after_update = true

  description = "test raw image"
  tags = {
    ssh-user = "opensuse"
  }

  cpu    = 2
  memory = "2Gi"

  efi         = true
  secure_boot = true

  run_strategy = "RerunOnFailure"
  hostname     = "opensuse154"
  machine_type = "q35"

  ssh_keys = [
    harvester_ssh_key.mysshkey.id
  ]

  network_interface {
    name           = "nic-1"
    network_name   = harvester_network.cluster-vlan1.id
    wait_for_lease = true
  }

  network_interface {
    name         = "nic-2"
    model        = "virtio"
    type         = "bridge"
    network_name = harvester_network.cluster-vlan["2"].id
  }

  network_interface {
    name         = "nic-3"
    model        = "e1000"
    type         = "bridge"
    network_name = harvester_network.cluster-vlan["3"].id
  }

  disk {
    name       = "rootdisk"
    type       = "disk"
    size       = "10Gi"
    bus        = "virtio"
    boot_order = 1

    image       = harvester_image.opensuse154.id
    auto_delete = true
  }

  disk {
    name = "mount-disk"
    type = "disk"
    bus  = "scsi"

    existing_volume_name = harvester_volume.mount-disk.name
    auto_delete          = false
    hot_plug             = true
  }

  cloudinit {
    user_data_secret_name    = harvester_cloudinit_secret.cloud-config-opensuse154.name
    network_data_secret_name = harvester_cloudinit_secret.cloud-config-opensuse154.name
  }
}
```

<!-- schema generated by tfplugindocs -->
## Schema

### Required

- **disk** (Block List, Min: 1) (see [below for nested schema](#nestedblock--disk))
- **name** (String) A unique name
- **network_interface** (Block List, Min: 1) (see [below for nested schema](#nestedblock--network_interface))

### Optional

- **cloudinit** (Block List, Max: 1) (see [below for nested schema](#nestedblock--cloudinit))
- **cpu** (Number)
- **description** (String) Any text you want that better describes this resource
- **efi** (Boolean)
- **hostname** (String)
- **id** (String) The ID of this resource.
- **input** (Block List) (see [below for nested schema](#nestedblock--input))
- **machine_type** (String)
- **memory** (String)
- **namespace** (String)
- **reserved_memory** (String)
- **restart_after_update** (Boolean) restart vm after the vm is updated
- **run_strategy** (String) more info: https://kubevirt.io/user-guide/virtual_machines/run_strategies/
- **secure_boot** (Boolean) EFI must be enabled to use this feature
- **ssh_keys** (List of String)
- **start** (Boolean, Deprecated)
- **tags** (Map of String)
- **tpm** (Block List, Max: 1) (see [below for nested schema](#nestedblock--tpm))

### Read-Only

- **message** (String)
- **node_name** (String)
- **state** (String)

<a id="nestedblock--disk"></a>
### Nested Schema for `disk`

Required:

- **name** (String)

Optional:

- **access_mode** (String)
- **auto_delete** (Boolean)
- **boot_order** (Number)
- **bus** (String)
- **container_image_name** (String)
- **existing_volume_name** (String)
- **hot_plug** (Boolean)
- **image** (String)
- **size** (String)
- **storage_class_name** (String)
- **type** (String)
- **volume_mode** (String)
- **volume_name** (String)


<a id="nestedblock--network_interface"></a>
### Nested Schema for `network_interface`

Required:

- **name** (String)

Optional:

- **mac_address** (String)
- **model** (String)
- **network_name** (String) if the value is empty, management network is used
- **type** (String)
- **wait_for_lease** (Boolean) wait for this network interface to obtain an IP address. If a non-management network is used, this feature requires qemu-guest-agent installed and started in the VM, otherwise, VM creation will stuck until timeout

Read-Only:

- **interface_name** (String)
- **ip_address** (String)


<a id="nestedblock--cloudinit"></a>
### Nested Schema for `cloudinit`

Optional:

- **network_data** (String)
- **network_data_base64** (String)
- **network_data_secret_name** (String)
- **type** (String)
- **user_data** (String)
- **user_data_base64** (String)
- **user_data_secret_name** (String)


<a id="nestedblock--input"></a>
### Nested Schema for `input`

Required:

- **name** (String)

Optional:

- **bus** (String)
- **type** (String)


<a id="nestedblock--tpm"></a>
### Nested Schema for `tpm`

Optional:

- **name** (String) just add this field for doc generation

## Import

Import is supported using the following syntax:

```shell
terraform import harvester_virtualmachine.foo <Namespace>/<Name>
```
